/****************************************************************************
    Dimmer.c
    --------
    Use Atmel AT90S8535 to dim eight(or more) independent lamps
	CPU AT90S8535 by 4MHz
	
	quick & dirty
	
    Autor:   Markus Schiesser 
    Datum:   31.12.2004 
    Software: AVR-GCC
    
****************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

// 10ms(50Hz/2) / 0,25us (4MHz) = 40000 cycle
// 40000 cycle / 8(prescaler) = 5000 counts per 10ms
// 5000 counts / 100 = 50 
// 255 - 50 = 205 => 0xCD
// 0xCD + 0x07 (delays) = 0xD4
#define PRELOAD_TIMER  0xD4

char rx_value[8];
unsigned char i =0;
unsigned char counter=0;

// zero crossing detected
SIGNAL (SIG_INTERRUPT0)
{
	//turn on port B
	outp (0xFF,PORTB);
	
	//enable timer 0 overflow interrupt
	outp((1<<TOIE0), TIMSK);
	//preload timer 0
	outp(PRELOAD_TIMER, TCNT0); 
	//set prescaler to 8
	outp(2, TCCR0);
	//start counter
	counter = 0;
}

SIGNAL (SIG_OVERFLOW0)
{
	//reload timer 0
	outp(PRELOAD_TIMER, TCNT0);
	
	//increase counter
	counter ++;
		
	//check array with counter value
	if (counter == rx_value[0])
    {
		cbi (PORTB,0);
	}	
	
	if (counter == rx_value[1])
    {
		cbi (PORTB,1);
	}	
	
	if (counter == rx_value[2])
    {
		cbi (PORTB,2);
	}	
	
	if (counter == rx_value[3])
    {		
		cbi (PORTB,3);
	}
	
	if (counter == rx_value[4])
    {
		cbi (PORTB,4);
	}
	
	if (counter == rx_value[5])
    {
		cbi (PORTB,5);
	}
	
	if (counter == rx_value[6])
    {
		cbi (PORTB,6);
	}
	
	if (counter == rx_value[7])
    {
		cbi (PORTB,7);
	}
	
	
}		


SIGNAL (SIG_UART_RECV)
{
	//receive 8 values an put into an arry
	rx_value[i] = UDR;
		
		if (++i == 8) {
			i = 0;
		}
}		
	
void init_uart(void)
{
	// turn on receive interrupt uart
	outp((1<<RXCIE)|(1<<RXEN),UCR);
	//serial speed 9600 baud by 4MHz */
	outp(25, UBRR);  
	
	return;
}

void init_external_interrupt(void)
{
	//turn on external interrupt 0
	outp((1<<INT0),GIMSK);
	//interrupt on a falling edge
	outp((1<<ISC01)|(1<<ISC11),MCUCR);
	
	return;
}	

int main(void)
{
//port B output 
outp(0xff,DDRB);
//port D input (interrupt from zero detection)
outp(0x00,DDRD);

//init
init_uart ();
init_external_interrupt ();

//turn on general interrupt
sei(); 

for(;;){}
}